"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const configurationHelpers_1 = require("../configurationHelpers");
const constants_1 = require("../constants");
const extensionService_1 = require("./extensionService");
const serverProxy_1 = require("./serverProxy");
class AlDefinitionProvider extends extensionService_1.ExtensionService {
    constructor(context, languageServerClient, launchService) {
        super(context);
        this.languageServerClient = languageServerClient;
        this.launchService = launchService;
        this.serverProxy = new serverProxy_1.ServerProxy(this.languageServerClient);
    }
    activate() {
        this.registerForDisposal(vscode.languages.registerDefinitionProvider(constants_1.AlLanguageId, this));
    }
    provideDefinition(textDocument, position, token) {
        const configurations = this.launchService.getConfigurations();
        // To avoid an annoying behavior on Ctrl+Click, reference content is downloaded only when there is a single server configuration in launch.json.
        // A better way would to implement the "set default server" feature to choose a configuration once and persist the choice.
        let chosenConfiguration = null;
        if (configurations.length === 1) {
            chosenConfiguration = configurations[0];
        }
        const params = configurationHelpers_1.getAlParams();
        params.textDocumentPositionParams = this.languageServerClient.asTextDocumentPositionParams(textDocument, position);
        return this.serverProxy.sendRequest(chosenConfiguration, constants_1.AlGotoDefinitionRequest, params).then((resultParam) => {
            if (resultParam) {
                const uri = vscode.Uri.parse(resultParam.uri);
                const start = new vscode.Position(resultParam.range.start.line, resultParam.range.start.character);
                const end = new vscode.Position(resultParam.range.end.line, resultParam.range.end.character);
                return new vscode.Location(uri, new vscode.Range(start, end));
            }
            return undefined;
        });
    }
}
exports.AlDefinitionProvider = AlDefinitionProvider;
//# sourceMappingURL=alDefinitionProvider.js.map